@@ -23,7 +23,10 @@ import h5py                    # type: ignore
 import numpy                   # type: ignore
 from datetime import datetime
 from pandas import DataFrame   # type: ignore
-import dateutil.parser
+from dateutil.parser import parse
+from requests import get
+from requests.exceptions import MissingSchema
+from tempfile import NamedTemporaryFile
 import sys
 if sys.version_info >= (3, 8):
     from typing import Final, Any
@@ -45,7 +48,18 @@ class HDF5LabNotebook(LabNotebook):
     different types when read than when acquired. (See :ref:`hdf5-type-management`
     for details.)
 
-    :param name: HDF5 file to persist the notebook to
+    The name of the notebook can be a file or a URL. Only files can be created
+    or updated: if a URL is provided then the notebook will be read and
+    immediately marked as locked. This implies that ``create=True`` won't
+    work in conjunction with URLs.
+
+    .. important ::
+
+        Note that because of the design of the ``requests`` library used for
+        handling URLs, using a ``file:``-schema URL will result in an exception
+        being raised. Use filenames for accessing files.
+
+    :param name: HDF5 file or URL backing the notebook
     :param create: (optional) if True, erase any existing file (defaults to False)
     :param description: (optional) free text description of the notebook 
     '''
@@ -60,22 +74,31 @@ class HDF5LabNotebook(LabNotebook):
     RESULTS_DATASET : Final[str] = 'results'         #: Name of results dataset within the HDF5 group for a result set.
     PENDINGRESULTS_DATASET : Final[str] = 'pending'  #: Name of pending results dataset within the HDF5 group for a result set.
 
-    # Metadata for the notebook
+    # Metadata for the notebook and/or result sets
     CREATOR : Final[str] = 'creator'                 #: Attribute holding library information and URL.
     VERSION : Final[str] = 'version'                 #: Attribute holding the version of file structure used.
     DESCRIPTION : Final[str] = 'description'         #: Attribute holding the notebook and result set descriptions.
     CURRENT : Final[str] = 'current-resultset'       #: Attribute holding the tag of the current result set.
-
-    # Metadata for result sets
-    LOCKED : Final[str] = 'locked'                   #: Attribute flagging as result set as being locked to further changes.
+    LOCKED : Final[str] = 'locked'                   #: Attribute flagging as result set or notebook as being locked to further changes.
 
     def __init__(self, name : str, create : bool =False, description : str =None):
         # create an empty file structure
-        self._file : h5py.File = None          # file handle for underlying HDF5 file
+        self._file : h5py.File = None          # HDF5 file for underlying data
         self._group : h5py.Group = None        # group associated with the current result set
 
+        # if we're looking at a URL, some behaviour is different
+        try:
+            # attempt to open the name as a URL
+            response = get(name)
+
+            # if we get here, the name is a URL
+            self._isRemote = True
+        except MissingSchema:
+            # name is a file
+            self._isRemote = False
+
         # check for file needing creation
-        created = create or not os.path.isfile(name)
+        created = (not self._isRemote) and (create or not os.path.isfile(name))
         if created:
             self._create(name)                
 
@@ -103,6 +126,13 @@ class HDF5LabNotebook(LabNotebook):
 
     def commit(self):
         """Persist any changes in the result sets in the notebook to disc."""
+        if not self.isLocked():
+            self._open()
+            self._save()
+            self._close()
+
+    def _commit(self):
+        '''If we're finishing, commit regardless of the lock status.'''
         self._open()
         self._save()
         self._close()
@@ -146,7 +176,21 @@ class HDF5LabNotebook(LabNotebook):
     def _open(self):
         '''Open the HDF5 file that backs this notebook.'''
         if self._file is None:
-            self._file = h5py.File(self.name(), 'a')
+            if self._isRemote:
+                # open as a URL
+                response = get(self.name())
+
+                # slurp the contents into a local temp file
+                tf = NamedTemporaryFile(delete=False)
+                for chunk in response.iter_content(chunk_size=1024):
+                    tf.write(chunk)
+                tf.close()
+
+                # open the file read-only
+                self._file = h5py.File(tf.name, 'r')   # can only be read
+            else:
+                # open as a file 
+                self._file = h5py.File(self.name(), 'a')
 
     def _close(self):
         '''Close the underlying HDF5 file.'''
@@ -154,6 +198,10 @@ class HDF5LabNotebook(LabNotebook):
             self._file.close()
             self._file = None
 
+        # if we loaded from a URL, lock the notebook
+        if self._isRemote:
+            self.finish(commit=False)   # don't try to commit it when finishing
+
     def _write(self, tag : str):
         '''Write the given result set to the file.
 
@@ -328,7 +376,7 @@ class HDF5LabNotebook(LabNotebook):
                         if k in [ Experiment.START_TIME, Experiment.END_TIME ]:
                             try:
                                 # patch known datestamps to datetime objects
-                                dt = dateutil.parser.parse(entry[j])
+                                dt = parse(entry[j])
                                 entry[j] = dt
                             except:
                                 # leave unchanged if we can't patch
@@ -368,7 +416,7 @@ class HDF5LabNotebook(LabNotebook):
                 # record the pending result
                 self.addPendingResult(params, jobid, tag)
 
-        # lock the result set if flagged
+        # lock the result set if flagged 
         if locked:
             rs.finish()
 
@@ -390,12 +438,14 @@ class HDF5LabNotebook(LabNotebook):
         meta[self.VERSION] = self.Version
         meta[self.DESCRIPTION] = self.description()
         meta[self.CURRENT] = self.resultSetTag(self.current())
+        meta[self.LOCKED] = self.isLocked()
 
     def _load(self):
         '''Load the notebook and all result sets.'''
+        meta = self._file.attrs
 
         # version check
-        v = self._file.attrs[self.VERSION]
+        v = meta[self.VERSION]
         if v != self.Version:
             # at present can't handle different file structure versions
             raise NotebookVersionException(self.Version, v)
@@ -410,11 +460,15 @@ class HDF5LabNotebook(LabNotebook):
             self._read(tag)
 
         # read the current result set and select it
-        c = self._file.attrs[self.CURRENT]
+        c = meta[self.CURRENT]
         super(HDF5LabNotebook, self).select(c)
 
         # read other attributes
-        self._description = self._file.attrs[self.DESCRIPTION]
+        self._description = meta[self.DESCRIPTION]
+
+        # lock if necessary
+        if meta[self.LOCKED]:
+            self.finish(commit=False)
 
 
     # ---------- Type conversion ----------
