@@ -70,6 +70,15 @@ class NotebookVersionException(Exception):
         :returns: the actual version'''
         return self._expected
 
+class LabNotebookLockedException(Exception):
+    '''An exception raised if an attempt is made to write to a notebook
+    that's been locked by a call to :meth:`LabNotebook.finish`. This includes
+    attemoting to add result sets.'''
+
+    def __init__(self):
+        super(LabNotebookLockedException, self).__init__('Lab notebook locked')
+
+
 
 class LabNotebook(object):
     '''A "laboratory notebook" collecting together the results obtained from
@@ -88,7 +97,7 @@ class LabNotebook(object):
     # Defaults
     DEFAULT_RESULTSET : Final[str] = 'epyc.resultset.default'  #: Tag for the default result set.
 
-    def __init__(self, name : str =None, description : str =None):
+    def __init__(self, name : str ='', description : str =None):
         if description is None:
             description = 'A notebook' 
         self._name : Optional[str] = name                    # name
@@ -96,6 +105,7 @@ class LabNotebook(object):
         self._resultSets: Dict[str, ResultSet] = dict()      # tag to result set
         self._resultSetTags : Dict[ResultSet, str] = dict()  # result set to tag
         self._pending : Dict[str, ResultSet] = dict()        # pending results job ids to result sets
+        self._locked : bool = False                          # locked flag
 
         # add a result set with the default tag, and make it current
         defrc = self.addResultSet(self.DEFAULT_RESULTSET)
@@ -105,7 +115,7 @@ class LabNotebook(object):
 
     # ---------- Access ----------
 
-    def name(self) -> Optional[str]:
+    def name(self) -> str:
         """Return the name of the notebook. If the notebook is persistent,
         this likely relates to its storage in some way (for example a
         file name).
@@ -123,6 +133,7 @@ class LabNotebook(object):
         '''Set the free text description of the notebook.
 
         :param d: the description'''
+        self.assertUnlocked()
         self._description = d
 
 
@@ -140,6 +151,11 @@ class LabNotebook(object):
         automatically in some sub-classes, depending on their implementation."""
         pass
 
+    def _commit(self):
+        '''Private API for forcing commits, called when finishing a notebook.
+        Defaults to the same behaviour as  :meth:`commit`.'''
+        self.commit()
+
 
     # ---------- Managing results sets ----------
 
@@ -150,6 +166,7 @@ class LabNotebook(object):
         :param tag: unique tag for this result set
         :param: (optional) free text description of the result set
         :returns: the result set'''
+        self.assertUnlocked()
         rs = ResultSet(description)
         self._resultSets[tag] = rs
         self._resultSetTags[rs] = tag
@@ -207,10 +224,43 @@ class LabNotebook(object):
         '''Return the number of result sets in this notebook.
         Same as :meth:`numberOfResultSets`.
 
-        :returns: the numbernof result sets'''
+        :returns: the number of result sets'''
         return self.numberOfResultSets()
 
 
+    # ---------- Finishing ----------
+
+    def finish(self, commit=True):
+        '''Mark the entire notebook as finished, closing and locking all result sets
+        against further changes. Finishing a persistent notebook commits it.
+        
+        By default the finished notebook is committed as such. In certain cases
+        it may be desirable to finish the notebook but not commit it, *i.e.*, to
+        stop updates in memory without changing the backing file. Setting ``commit=False``
+        will accomplish this.
+        
+        :param commit: (optional) commit the notebook (defaults to True)'''
+        if not self.isLocked():
+            for tag in self.resultSets():
+                rs = self.resultSet(tag)
+                rs.finish()
+            self._locked = True   # set here to make sure the status is committed
+            if commit:
+                self._commit()
+
+    def isLocked(self) -> bool:
+        '''Returns true if the notebook is locked.
+
+        :returns: True if the notebook is locked'''
+        return self._locked
+
+    def assertUnlocked(self):
+        '''Tests whether the notebook is locked, and raises a :class:`LabNotebookLockedException`
+        if so.'''
+        if self.isLocked():
+            raise LabNotebookLockedException()
+
+
     # ---------- Managing pending results in the current result set ----------
 
     def addPendingResult(self, params : Dict[str, Any], jobid : str, tag : str =None):
@@ -222,6 +272,7 @@ class LabNotebook(object):
         :param params: the experimental parameters
         :param jobid: the job id
         :param tag: (optional) the tag of the result set receiving the pending result (defaults to the current result set)'''
+        self.assertUnlocked()
         if tag is None:
             rs = self._current
         else:
@@ -247,6 +298,7 @@ class LabNotebook(object):
 
         :param rc: the results dict
         :param jobid: the job id'''
+        self.assertUnlocked()
 
         # add the result to the correct result set
         tag = self._resultSetTags[self._pending[jobid]]
@@ -267,6 +319,7 @@ class LabNotebook(object):
         the result set that is selected as current.
 
         :param jobid: the job id'''
+        self.assertUnlocked()
 
         # find the result set for the job
         rs = self._pending[jobid]
