@@ -22,6 +22,8 @@ import unittest
 import os
 from tempfile import NamedTemporaryFile
 
+# Remote HDF5 file for testiong URL behaviour
+testFileURL = 'https://raw.githubusercontent.com/simoninireland/epyc/dev/test/test.h5'
 
 class SampleExperiment(Experiment):
     '''A very simple experiment that adds up its parameters.'''
@@ -574,7 +576,7 @@ class HDF5LabNotebookTests(unittest.TestCase):
             self.assertEqual(nb.numberOfResults(), 0)
             self.assertEqual(nb.numberOfPendingResults(), 0)
 
-    def testLocking(self):
+    def testLockingResultSets(self):
         '''Test that a locked result set stays locked.'''
         e = SampleExperiment()
         nb = HDF5LabNotebook(self._fn, create=True)
@@ -607,6 +609,73 @@ class HDF5LabNotebookTests(unittest.TestCase):
         nb1.select(LabNotebook.DEFAULT_RESULTSET)
         self.assertFalse(nb1.current().isLocked())
 
+    def testLockingNotebook(self):
+        '''Test notebook locking.'''
+        e = SampleExperiment()
+        nb = HDF5LabNotebook(self._fn, create=True)
+
+        rc1 = e.set(dict(k=10)).run()
+        rc2 = e.set(dict(k=20)).run()
+        rc3 = e.set(dict(k=30)).run()
+        rc4 = e.set(dict(k=40)).run()
+
+        # first result set
+        nb.addResultSet('first')
+        nb.addResult(rc1)
+        nb.addResult(rc2)
+
+        # second
+        nb.addResultSet('second')
+        nb.addResult(rc3)
+        nb.addPendingResult(rc4[Experiment.PARAMETERS], '2345')
+
+        # lock the notebook
+        nb.finish()
+
+        # check we can't add new result sets
+        with self.assertRaises(LabNotebookLockedException):
+            nb.addResultSet('third')
+
+        # check the notebook in still  locked when reloaded
+        nb.commit()
+        with HDF5LabNotebook(self._fn).open() as nb1:
+            self.assertTrue(nb1.isLocked())
+            with self.assertRaises(LabNotebookLockedException):
+                nb.addResultSet('third')
+            rs = nb1.select('first')
+            self.assertTrue(rs.isLocked())
+            self.assertEqual(rs.numberOfResults(), 2)
+            rs = nb1.select('second')
+            self.assertTrue(rs.isLocked())
+            self.assertEqual(rs.numberOfPendingResults(), 0)
+            self.assertEqual(rs.numberOfResults(), 2)
+            rcs = nb1.resultsFor(rc4[Experiment.PARAMETERS])
+            self.assertEqual(len(rcs), 1)
+        
+    @unittest.skip('Seems to depend on exact config of the underlying filesystem')
+    def testNoWriteWhenLocked(self):
+        '''Test that no writing happens after a notebook is locked.'''
+        nb = HDF5LabNotebook(self._fn, create=True)
+
+        # commit the finished notebook
+        nb.addResultSet('first')
+        nb.finish()
+        t1 = os.path.getmtime(self._fn)
+
+        # reload and commit again, which shouldn't write anything
+        with HDF5LabNotebook(self._fn).open() as nb1:
+            nb1.commit()
+        t2 = os.path.getmtime(self._fn)
+
+        self.assertEqual(t1, t2)
+
+
+    def testLoadingFromURL(self):
+        '''Test we can load a read-only notebook from a URL.'''
+        with HDF5LabNotebook(testFileURL).open() as nb1:
+            self.assertIn('first', nb1.resultSets())
+            self.assertTrue(nb1.isLocked())
+
 if __name__ == '__main__':
     unittest.main()
 
